import re
from typing import Dict, List
from dataclasses import dataclass


def get_values(output_str: str) -> Dict[str, str]:
    """
    Get the dictionary representing actual value passed to the function
    """
    res_d = {}
    for line in output_str.splitlines():
        m = re.match(r'(\w+)=(.+)', line.strip())
        assert m is not None
        res_d[m.group(1)] = m.group(2)
    return res_d


@dataclass(frozen=True)
class HelpOption:
    name: str
    line: str


def get_options(help_str: str) -> List[HelpOption]:
    """
    Get the option names obtained from the help generated by glacier.

    The output will be as follows

    Options:
      --name TEXT                     Name of this operation.  [required]
      --env [development|production]  Specifying this operation is for whether dev or prod.  [required]  # noqa
      --verbose                       Verbose output will be shown if set.
      -h, --help                      Show this message and exit.
    """
    options_found = False
    res: List[HelpOption] = []
    for line in help_str.splitlines():
        if line.startswith('Options'):
            options_found = True
            continue
        if options_found:
            if line.startswith('  '):
                m = re.search(r'^  --([\w-]+)', line)
                if m is None:
                    continue
                option_name = m.group(1)
                res.append(
                    HelpOption(
                        name=option_name,
                        line=line.strip(),
                    )
                )
            else:
                options_found = False
    return res



from enum import Enum

from glacier import glacier


class Env(Enum):
    DEV = 'development'
    PROD = 'production'


def main(
    _path: str,
    name: str,
    env: Env,
    verbose: bool = False,
) -> None:
    """
    This is my simple entry point of CLI.

    Args:
        _path: Positional argument representing the target file path.
        name: Name of this operation.
        env: Specifying this operation is for whether dev or prod.
        verbose: Verbose output will be shown if set.
    """
    print(_path)
    print(name)
    print(env)
    print(verbose)
    return


if __name__ == '__main__':
    glacier(main)




from glacier import glacier


def main() -> None:
    return


def sub_1() -> None:
    return


def sub_2() -> None:
    return


if __name__ == '__main__':
    glacier({
        'main': main,
        'sub': [
            sub_1,
            sub_2,
        ],
    })



from enum import Enum

from glacier import glacier


def main_google(
    _path: str,
    name: str,
    verbose: bool = False,
) -> None:
    """
    This is my simple entry point of CLI.

    Args:
        _path: Positional argument representing the target file path.
        name: Name of this operation.
        verbose: Verbose output will be shown if set.
    """
    print(_path)
    print(name)
    print(verbose)
    return


def main_numpy(
    _path: str,
    name: str,
    verbose: bool = False,
) -> None:
    """
    This is my simple entry point of CLI.

    Parameters
    ----------
    _path: str
        Positional argument representing the target file path.
    name: str
        Name of this operation.
    verbose: bool
        Verbose output will be shown if set.
    """
    print(_path)
    print(name)
    print(verbose)
    return


def main_restructured_text(
    _path: str,
    name: str,
    verbose: bool = False,
) -> None:
    """
    This is my simple entry point of CLI.

    :param _path: Positional argument representing the target file path.
    :param name: Name of this operation.
    :param verbose: Verbose output will be shown if set.
    """
    print(_path)
    print(name)
    print(verbose)
    return


if __name__ == '__main__':
    glacier({
        'google': main_google,
        'numpy': main_numpy,
        'restructured-text': main_restructured_text,
    })



import unittest

from glacier.docstring import Arg, Doc, GoogleParser, NumpyParser

from glacier.docstring import RestructuredTextParser


class TestArg(unittest.TestCase):
    def test_arg_of_lines_google(self) -> None:
        arg_lines = [
            'hoge: description of hoge',
            '      that can be multilined.',
            'fuga: description of fu-',
            '      ga.',
        ]
        args = Arg.of_lines_google(arg_lines)
        assert args == [
            Arg(name='hoge', description='description of hoge that can be multilined.'),
            Arg(name='fuga', description='description of fuga.'),
        ]

    def test_arg_of_lines_numpy(self) -> None:
        arg_lines = [
            'hoge: str',
            '      description of hoge following type',
            'fuga:',
            '      description of fuga',
        ]
        args = Arg.of_lines_numpy(arg_lines)
        assert args == [
            Arg(name='hoge', description='description of hoge following type'),
            Arg(name='fuga', description='description of fuga'),
        ]

    def test_arg_of_lines_resttxt(self) -> None:
        arg_lines = [
            ':param hoge: description of hoge.',
            ':param fuga: description of fuga',
            '             multilined.',
        ]
        args = Arg.of_lines_resttxt(arg_lines)
        assert args == [
            Arg(name='hoge', description='description of hoge.'),
            Arg(name='fuga', description='description of fuga multilined.'),
        ]


class TestDocstringGoogle(unittest.TestCase):
    def test_google_one_line_description(self) -> None:
        docstring = """ This is oneline docstring. """
        parser = GoogleParser()
        doc = parser.parse(docstring)
        assert doc == Doc(
            description='This is oneline docstring.',
            args=[],
        )
        return

    def test_google_simple(self) -> None:
        docstring = """
        This is a simple docstring.

        Args:
            foo: Description of foo.
            bar: Description of bar.
        """
        parser = GoogleParser()
        doc = parser.parse(docstring)
        assert doc == Doc(
            description='This is a simple docstring.',
            args=[
                Arg(name='foo', description='Description of foo.'),
                Arg(name='bar', description='Description of bar.'),
            ],
        )
        return


class TestDocstringNumpy(unittest.TestCase):
    def test_numpy_one_line_description(self) -> None:
        docstring = """ This is oneline docstring. """
        parser = NumpyParser()
        doc = parser.parse(docstring)
        assert doc == Doc(
            description='This is oneline docstring.',
            args=[],
        )
        return

    def test_numpy_simple(self) -> None:
        docstring = """
        This is a simple docstring.

        Parameters
        ----------
        foo: str
             Description of foo.
        bar: int
             Description of bar.
        """
        parser = NumpyParser()
        doc = parser.parse(docstring)
        assert doc == Doc(
            description='This is a simple docstring.',
            args=[
                Arg(name='foo', description='Description of foo.'),
                Arg(name='bar', description='Description of bar.'),
            ],
        )
        return


class TestDocstringRestructuredText(unittest.TestCase):
    def test_resttext_one_line_description(self) -> None:
        docstring = """ This is oneline docstring. """
        parser = NumpyParser()
        doc = parser.parse(docstring)
        assert doc == Doc(
            description='This is oneline docstring.',
            args=[],
        )
        return

    def test_resttext_simple(self) -> None:
        docstring = """
        This is a simple docstring.

        :param foo: Description of foo.
        :param bar: Description of bar
                    that is multilined.
        """
        parser = RestructuredTextParser()
        doc = parser.parse(docstring)
        assert doc == Doc(
            description='This is a simple docstring.',
            args=[
                Arg(name='foo', description='Description of foo.'),
                Arg(name='bar', description='Description of bar that is multilined.'),
            ],
        )
        return



from glacier import glacier


def run(name: str, verbose: bool = False) -> None:
    """ Run """
    pass


def build(name: str, verbose: bool = False) -> None:
    """ Build """
    pass


def test(name: str, verbose: bool = False) -> None:
    """ Test """
    return


if __name__ == '__main__':
    glacier([run, build, test])



import unittest
from enum import Enum

from click.testing import CliRunner

from glacier.core import _get_click_command, glacier_group
from tests.utils import get_options, get_values


class Env(Enum):
    DEV = 'development'
    PROD = 'production'


def my_function_google(
    _path: str,
    name: str,
    age: int,
    is_test: bool,
    env: Env,
    verbose: bool = False,
) -> None:
    """
    This is my test function for generating CLI entrypoint.

    Args:
        _path: path input
        name: Name of the user.
        age: Age of the user.
        is_test: whether it is test or not.
        env: Environment where to run the CLI.
        verbose: If set, verbose output will be shown.
    """
    # Check if all arguments have the property type.
    assert type(_path) == str
    assert type(name) == str
    assert type(age) == int
    assert type(is_test) == bool
    assert isinstance(env, Env)
    assert type(verbose) == bool

    # Print all values
    print(f'_path={_path}')
    print(f'name={name}')
    print(f'age={age}')
    print(f'is_test={is_test}')
    print(f'env={env}')
    print(f'verbose={verbose}')
    return


def my_function_numpy_docstring(
    _path: str,
    name: str,
    age: int,
    is_test: bool,
    env: Env,
    verbose: bool = False,
) -> None:
    """
    This is my test function for generating CLI entrypoint.

    Parameters
    ----------
    _path: str
        path input
    name: str
        Name of the user.
    age: int
        Age of the user.
    is_test: bool
        whether it is test or not.
    env: Env
        Environment where to run the CLI.
    verbose: bool
        If set, verbose output will be shown.
    """
    # Check if all arguments have the property type.
    assert type(_path) == str
    assert type(name) == str
    assert type(age) == int
    assert type(is_test) == bool
    assert isinstance(env, Env)
    assert type(verbose) == bool

    # Print all values
    print(f'_path={_path}')
    print(f'name={name}')
    print(f'age={age}')
    print(f'is_test={is_test}')
    print(f'env={env}')
    print(f'verbose={verbose}')
    return


def my_function_restructured_text_docstring(
    _path: str,
    name: str,
    age: int,
    is_test: bool,
    env: Env,
    verbose: bool = False,
) -> None:
    """
    This is my test function for generating CLI entrypoint.

    :param _path: path input
    :param name: Name of the user.
    :param age: Age of the user.
    :param is_test: whether it is test or not.
    :param env: Environment where to run the CLI.
    :param verbose: If set, verbose output will be shown.
    """
    # Check if all arguments have the property type.
    assert type(_path) == str
    assert type(name) == str
    assert type(age) == int
    assert type(is_test) == bool
    assert isinstance(env, Env)
    assert type(verbose) == bool

    # Print all values
    print(f'_path={_path}')
    print(f'name={name}')
    print(f'age={age}')
    print(f'is_test={is_test}')
    print(f'env={env}')
    print(f'verbose={verbose}')
    return


class TestCore(unittest.TestCase):
    def test_glacier_ok(self) -> None:
        """
        Appropriate pattern (perfectly happy path).
        """
        for function in [
            my_function_google,
            my_function_numpy_docstring,
            my_function_restructured_text_docstring,
        ]:
            f = _get_click_command(function)
            runner = CliRunner()

            # All required arguments are provided, and
            # verbose (default value is set) is omitted.
            result = runner.invoke(
                f,
                [
                    'path',
                    '--name=taro',
                    '--age=10',
                    '--is-test',
                    '--env=development',
                ],
            )
            # No excption occurs
            assert not result.exception

            # Get output
            res_d = get_values(result.output)
            assert res_d['_path'] == 'path'
            assert res_d['name'] == 'taro'
            assert res_d['age'] == '10'
            assert res_d['is_test'] == 'True'
            assert res_d['env'] == 'Env.DEV'
            assert res_d['verbose'] == 'False'
        return

    def test_glacier_help(self) -> None:
        """
        Check if help of CLI is correct.
        """
        for function in [
            my_function_google,
            my_function_numpy_docstring,
            my_function_restructured_text_docstring,
        ]:
            f = _get_click_command(function)
            runner = CliRunner()
            result = runner.invoke(
                f,
                [
                    '-h',
                ],
            )
            # No exception occurs.
            assert not result.exception
            # Assert that docstring description is contained in help.
            assert 'This is my test function for generating CLI entrypoint.' in result.output

            # Assert that options are displayed in order.
            help_options = get_options(result.output)

            # Assert the name (and its order) of options
            assert help_options[0].name == 'name'
            assert help_options[1].name == 'age'
            assert help_options[2].name == 'is-test'
            assert help_options[3].name == 'env'
            assert help_options[4].name == 'verbose'

            # Assert that desired description is included in each line
            assert 'Name of the user' in help_options[0].line
            assert 'Age of the user.' in help_options[1].line
            assert 'whether it is test or not.' in help_options[2].line
            assert 'Environment where to run the CLI.' in help_options[3].line
            assert 'If set, verbose output will be shown.' in help_options[4].line
        return

    def test_glacier_nestes_groups(self) -> None:
        """
        Check if CLI with nested click.group works correctly.
        """

        def a_1() -> None:
            print('a_1')

        def a_2() -> None:
            print('a_2')

        def b() -> None:
            print('b')

        f = glacier_group(
            {
                'a': [
                    a_1,
                    a_2,
                ],
                'b': b,
            }
        )
        runner = CliRunner()
        assert (
            runner.invoke(
                f,
                [
                    'a',
                    'a-1',
                ],
            ).output
            == 'a_1\n'
        )
        assert (
            runner.invoke(
                f,
                [
                    'a',
                    'a-2',
                ],
            ).output
            == 'a_2\n'
        )
        assert (
            runner.invoke(
                f,
                [
                    'b',
                ],
            ).output
            == 'b\n'
        )
        return



from glacier import glacier


def f1(name: str, verbose: bool = False) -> None:
    pass


def f2(name: str, verbose: bool = False) -> None:
    pass


def f3(name: str, verbose: bool = False) -> None:
    pass


if __name__ == '__main__':
    glacier({
        'run': f1,
        'build': f2,
        'test': f3,
    })
